@@ -140,10 +140,12 @@ type ClusterConfiguration struct {
 	// +k8s:conversion-gen=false
 	CIImageRepository string
 
-	// FeatureGates enabled by the user.
+	// FeatureGates holds kubeadm-specific feature gates enabled by the user.
 	FeatureGates map[string]bool
 
-	// The cluster name
+	// The cluster name. This name will be used in kubeconfig files generated by kubeadm
+	// and will also be passed as a value to the kube-controller-manager's --cluster-name flag.
+	// Default value is 'kubernetes'.
 	ClusterName string
 
 	// EncryptionAlgorithm holds the type of asymmetric encryption algorithm used for keys and certificates.
@@ -286,7 +288,8 @@ type Etcd struct {
 
 // LocalEtcd describes that kubeadm should run an etcd cluster locally
 type LocalEtcd struct {
-	// ImageMeta allows to customize the container used for etcd
+	// ImageMeta allows to customize the container image used for etcd. Passing a custom etcd image
+	// tells kubeadm upgrade that this image is user-managed and that its upgrade must be skipped.
 	ImageMeta `json:",inline"`
 
 	// DataDir is the directory etcd will place its data.
@@ -313,10 +316,23 @@ type LocalEtcd struct {
 
 // ExternalEtcd describes an external etcd cluster
 type ExternalEtcd struct {
-
-	// Endpoints of etcd members. Useful for using external etcd.
-	// If not provided, kubeadm will run etcd in a static pod.
+	// Endpoints of etcd members. Required when using external etcd.
+	// Specifies the client URLs (usually gRPC endpoints) for etcd communication.
+	// By default, these endpoints handle both gRPC traffic (primary etcd protocol)
+	// and HTTP traffic (metrics, health checks). However, if HTTPEndpoints is configured,
+	// the gRPC and HTTP traffic can be separated for better security and performance.
+	// Corresponds to etcd's --listen-client-urls configuration.
 	Endpoints []string
+
+	// HTTPEndpoints are the dedicated HTTP endpoints for etcd communication.
+	// When configured, HTTP traffic (such as /metrics and /health endpoints) is separated
+	// from the gRPC traffic handled by Endpoints. This separation allows for better access
+	// control, as HTTP endpoints can be exposed without exposing the primary gRPC interface.
+	// Corresponds to etcd's --listen-client-http-urls configuration.
+	// If not provided, Endpoints will be used for both gRPC and HTTP traffic.
+	// +optional
+	HTTPEndpoints []string
+
 	// CAFile is an SSL Certificate Authority file used to secure etcd communication.
 	CAFile string
 	// CertFile is an SSL certification file used to secure etcd communication.
