@@ -707,7 +707,20 @@ func allowRelaxedServiceNameValidation(oldIngress *networking.Ingress) bool {
 	if oldIngress == nil {
 		return false
 	}
-	// If feature gate is disabled, check if any service names in the old Ingresss
+
+	// Check if default backend service names in the old Ingress
+	if oldIngress.Spec.DefaultBackend != nil && oldIngress.Spec.DefaultBackend.Service != nil {
+		serviceName := oldIngress.Spec.DefaultBackend.Service.Name
+		// If a name doesn't validate with apimachineryvalidation.NameIsDNS1035Label, but does validate with apimachineryvalidation.NameIsDNSLabel,
+		// then we allow it to be used as a Service name in an Ingress.
+		dnsLabelValidationErrors := apimachineryvalidation.NameIsDNSLabel(serviceName, false)
+		dns1035LabelValidationErrors := apimachineryvalidation.NameIsDNS1035Label(serviceName, false)
+		if len(dnsLabelValidationErrors) == 0 && len(dns1035LabelValidationErrors) > 0 {
+			return true
+		}
+	}
+
+	// If feature gate is disabled, check if any service names in the old Ingress
 	for _, rule := range oldIngress.Spec.Rules {
 		if rule.HTTP == nil {
 			continue
